@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2011-2020 The Octave Project Developers
+// Copyright (C) 2011-2022 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -43,12 +43,9 @@
 
 #include "gui-preferences-ws.h"
 #include "octave-qobject.h"
+#include "octave-qtutils.h"
 #include "workspace-view.h"
 
-#include "interpreter-private.h"
-#include "interpreter.h"
-#include "syminfo.h"
-
 namespace octave
 {
   workspace_view::workspace_view (QWidget *p, base_qobject& oct_qobj)
@@ -120,16 +117,8 @@ namespace octave
           (settings->value (ws_column_state.key).toByteArray ());
 
         // Set header properties for sorting
-#if defined (HAVE_QHEADERVIEW_SETSECTIONSCLICKABLE)
         m_view->horizontalHeader ()->setSectionsClickable (true);
-#else
-        m_view->horizontalHeader ()->setClickable (true);
-#endif
-#if defined (HAVE_QHEADERVIEW_SETSECTIONSMOVABLE)
         m_view->horizontalHeader ()->setSectionsMovable (true);
-#else
-        m_view->horizontalHeader ()->setMovable (true);
-#endif
         m_view->horizontalHeader ()->setSortIndicator (
           settings->value (ws_sort_column).toInt (),
           static_cast<Qt::SortOrder> (settings->value (ws_sort_order).toUInt ()));
@@ -140,8 +129,8 @@ namespace octave
 
         m_view->horizontalHeader ()->setContextMenuPolicy (Qt::CustomContextMenu);
         connect (m_view->horizontalHeader (),
-                 SIGNAL (customContextMenuRequested (const QPoint &)),
-                 this, SLOT (header_contextmenu_requested (const QPoint &)));
+                 &QTableView::customContextMenuRequested,
+                 this, &workspace_view::header_contextmenu_requested);
 
         // Init state of the filter
         m_filter->addItems (settings->value (ws_mru_list.key).toStringList ());
@@ -154,25 +143,21 @@ namespace octave
 
     // Connect signals and slots.
 
-    connect (m_filter, SIGNAL (editTextChanged (const QString&)),
-             this, SLOT (filter_update (const QString&)));
-    connect (m_filter_checkbox, SIGNAL (toggled (bool)),
-             this, SLOT (filter_activate (bool)));
-    connect (m_filter->lineEdit (), SIGNAL (editingFinished ()),
-             this, SLOT (update_filter_history ()));
+    connect (m_filter, &QComboBox::editTextChanged,
+             this, &workspace_view::filter_update);
+    connect (m_filter_checkbox, &QCheckBox::toggled,
+             this, &workspace_view::filter_activate);
+    connect (m_filter->lineEdit (), &QLineEdit::editingFinished,
+             this, &workspace_view::update_filter_history);
 
-    connect (m_view, SIGNAL (customContextMenuRequested (const QPoint&)),
-             this, SLOT (contextmenu_requested (const QPoint&)));
+    connect (m_view, &QTableView::customContextMenuRequested,
+             this, &workspace_view::contextmenu_requested);
 
-    connect (m_view, SIGNAL (activated (QModelIndex)),
-             this, SLOT (handle_contextmenu_edit (void)));
+    connect (m_view, &QTableView::activated,
+             this, &workspace_view::handle_contextmenu_edit);
 
-    connect (this, SIGNAL (command_requested (const QString&)),
-             p, SLOT (execute_command_in_terminal (const QString&)));
-
-    connect (this,
-             SIGNAL (edit_variable_signal (const QString&, const octave_value&)),
-             p, SLOT (edit_variable (const QString&, const octave_value&)));
+    if (! p)
+      make_window ();
   }
 
   void workspace_view::setModel (workspace_model *model)
@@ -200,7 +185,7 @@ namespace octave
     m_model->notice_settings (settings); // update colors of model first
 
     for (int i = 0; i < ws_columns_shown.length (); i++)
-      m_view->setColumnHidden (i + 1, ! settings->value (ws_columns_shown_keys.at (i),true).toBool ());
+      m_view->setColumnHidden (i + 1, ! settings->value (ws_columns_shown_keys.at (i), true).toBool ());
 
     QString tool_tip;
 
@@ -217,7 +202,7 @@ namespace octave
               .arg (m_model->storage_class_color (i).name ())
               .arg (m_model->storage_class_color (i + ws_colors_count).name ())
               .arg (QCoreApplication::translate ("octave::settings_dialog",
-                                ws_color_names.at (i).toStdString ().data ()));
+                                                 ws_color_names.at (i).toStdString ().data ()));
           }
       }
 
@@ -254,6 +239,20 @@ namespace octave
     octave_dock_widget::save_settings ();
   }
 
+  void workspace_view::set_filter_focus (bool focus)
+  {
+    if (focus)
+      {
+        m_filter->setFocus ();
+        setFocusProxy (m_filter);
+      }
+    else
+      {
+        m_view->setFocus ();
+        setFocusProxy (m_view);
+      }
+  }
+
   void
   workspace_view::filter_update (const QString& expression)
   {
@@ -271,6 +270,8 @@ namespace octave
       filter_update (m_filter->currentText ());
     else
       filter_update (QString ());
+
+    set_filter_focus (state);
   }
 
   void
@@ -302,9 +303,23 @@ namespace octave
                             &sig_mapper, SLOT (map ()));
         sig_mapper.setMapping (action, i);
         action->setCheckable (true);
-        action->setChecked (settings->value (ws_columns_shown_keys.at (i),true).toBool ());
+        action->setChecked (settings->value (ws_columns_shown_keys.at (i), true).toBool ());
       }
 
+    // FIXME: We could use
+    //
+    //   connect (&m_sig_mapper, QOverload<int>::of (&QSignalMapper::mapped),
+    //            this, &workspace_view::toggle_header);
+    //
+    // but referring to QSignalMapper::mapped will generate deprecated
+    // function warnings from GCC.  We could also use
+    //
+    //   connect (&m_sig_mapper, &QSignalMapper::mappedInt,
+    //            this, &workspace_view::toggle_header);
+    //
+    // but the function mappedInt was not introduced until Qt 5.15 so
+    // we'll need a feature test.
+
     connect (&sig_mapper, SIGNAL (mapped (int)),
              this, SLOT (toggle_header (int)));
 
@@ -318,7 +333,7 @@ namespace octave
     gui_settings *settings = rmgr.get_settings ();
 
     QString key = ws_columns_shown_keys.at (col);
-    bool shown = settings->value (key,true).toBool ();
+    bool shown = settings->value (key, true).toBool ();
 
     m_view->setColumnHidden (col + 1, shown);
 
@@ -341,16 +356,17 @@ namespace octave
         QString var_name = get_var_name (index);
 
         menu.addAction (tr ("Open in Variable Editor"), this,
-                        SLOT (handle_contextmenu_edit ()));
+                        &workspace_view::handle_contextmenu_edit);
 
         menu.addAction (tr ("Copy name"), this,
-                        SLOT (handle_contextmenu_copy ()));
+                        &workspace_view::handle_contextmenu_copy);
 
         menu.addAction (tr ("Copy value"), this,
-                        SLOT (handle_contextmenu_copy_value ()));
+                        &workspace_view::handle_contextmenu_copy_value);
 
-        QAction *rename = menu.addAction (tr ("Rename"), this,
-                                          SLOT (handle_contextmenu_rename ()));
+        QAction *rename
+          = menu.addAction (tr ("Rename"), this,
+                            &workspace_view::handle_contextmenu_rename);
 
         // Use m_model here instead of using "m_view->model ()" because
         // that points to the proxy model.
@@ -361,18 +377,18 @@ namespace octave
           }
 
         menu.addAction ("Clear " + var_name, this,
-                        SLOT (handle_contextmenu_clear ()));
+                        &workspace_view::handle_contextmenu_clear);
 
         menu.addSeparator ();
 
         menu.addAction ("disp (" + var_name + ')', this,
-                        SLOT (handle_contextmenu_disp ()));
+                        &workspace_view::handle_contextmenu_disp);
 
         menu.addAction ("plot (" + var_name + ')', this,
-                        SLOT (handle_contextmenu_plot ()));
+                        &workspace_view::handle_contextmenu_plot);
 
         menu.addAction ("stem (" + var_name + ')', this,
-                        SLOT (handle_contextmenu_stem ()));
+                        &workspace_view::handle_contextmenu_stem);
 
         menu.addSeparator ();
 
@@ -380,10 +396,10 @@ namespace octave
 
     if (m_filter_shown)
       menu.addAction (tr ("Hide filter"), this,
-                      SLOT (handle_contextmenu_filter ()));
+                      &workspace_view::handle_contextmenu_filter);
     else
       menu.addAction (tr ("Show filter"), this,
-                      SLOT (handle_contextmenu_filter ()));
+                      &workspace_view::handle_contextmenu_filter);
 
     menu.exec (m_view->mapToGlobal (qpos));
   }
@@ -409,29 +425,7 @@ namespace octave
     QModelIndex index = m_view->currentIndex ();
 
     if (index.isValid ())
-      {
-        QString var_name = get_var_name (index);
-
-        emit interpreter_event
-          ([var_name] (interpreter& interp)
-           {
-             // INTERPRETER THREAD
-
-             octave_value val = interp.varval (var_name.toStdString ());
-
-             if (val.is_undefined ())
-               val = 0;
-
-             std::ostringstream buf;
-             val.print_raw (buf, true);
-
-             // FIXME: is the following operation thread safe or should
-             // it be done with a signal/slot connection?
-
-             QClipboard *clipboard = QApplication::clipboard ();
-             clipboard->setText (QString::fromStdString (buf.str ()));
-           });
-      }
+      emit copy_variable_value_to_clipboard (get_var_name (index));
   }
 
   void
@@ -464,14 +458,7 @@ namespace octave
     QModelIndex index = m_view->currentIndex ();
 
     if (index.isValid ())
-      {
-        QString var_name = get_var_name (index);
-
-        symbol_info_list syminfo = m_model->get_symbol_info ();
-        octave_value val = syminfo.varval (var_name.toStdString ());
-
-        emit edit_variable_signal (var_name, val);
-      }
+      emit edit_variable_signal (get_var_name (index));
   }
 
   void
@@ -503,6 +490,8 @@ namespace octave
   {
     m_filter_shown = ! m_filter_shown;
     m_filter_widget->setVisible (m_filter_shown);
+
+    set_filter_focus (m_filter_shown && m_filter_checkbox->isChecked ());
   }
 
   void
